//=============================================================================
// Popup Window MV v0.4b
//=============================================================================

/*:
 * @plugindesc Popup messages!
 * @author Vlue
 *
 * @param Popup Window Default Timer
 * @desc The default time a Popup Window stays active (in seconds)
 * @default 5
 *
 * @param Item Notification Default Opacity
 * @desc The default opacity of the window for item notifications
 * @default 175
 *
 * @param Item Notification Default Style
 * @desc Normal (Instant) or Typed (Letter by Letter)
 * @default Typed
 * 
 * @param Item Notification Message
 * @desc Format of the item notification message (icon, item, amount)
 * @default amountx icon item
 *
 * @param Automatic Popup on Gain Item
 * @desc Whether or not to automaticaly preform a popup when granted an item
 * @default True
 *
 * @param Display Position 
 * @desc Player, Center, TopLeft, Top, TopRight, Right, BottomRight, Bottom, BottomLeft, Left
 * @default Player
 *
 * @param Item Display Position
 * @desc Player, Center, TopLeft, Top, TopRight, Right, BottomRight, Bottom, BottomLeft, Left
 * @default Player
 *
 * @help Popup Window MV v0.4b
 *  Follow me on facebook: https://www.facebook.com/DaimoniousTailsGames/
 *   for the most recent updates!
 *  Find all of my work here: http://daimonioustails.weebly.com/
 *
 *   Plugin Commands:
 *    Popup position modifer 		    -Changes the position of the popup window
 *                                *Options include: Player, Center, TopLeft, Top, TopRight, 
 *                                                  Right, BottomRight, Bottom, BottomLeft, 
 *                                                  Left
 *    Popup automaticPopup true/false   -Enables/Disables automatic popups
 *
 *   Make custom popup messages whenever and wherever you like! How fancy.
 *   - Also automatically shows popup messages when an item is recieved.
 *
 *   To make a message popup use the script call:
 *    Popup.add(["text","text"...], time, opacity, style, x, y, position)
 *   Where "text" is any message (character codes allowed! just add an extra
 *    backslash). 
 *   Time is the time the popup remains active in seconds. (Can be closed manually
 *    with button press) (Can be omitted)
 *   X and Y are the offset of the window. Can be omitted and window will
 *    automatically position itself based on the position modifier.
 *   Window also automatically sizes itself to the width of the longest message.
 *   Opacity is the opacity of the window, can be omitted (defaults to 255)
 *   Style is the way the letters are displayed, either at once ("Normal") or 
 *    letter by letter ("Typed"), defaults to Normal
 *
 *   Examples:
 *    Popup.add(["Hello!"],5)
 *    Popup.add(["This is message number one!",
 *				"And this is message number two, right below one!"], 5)
 *    Popup.add(["\\i[20] Sword of Doom obtained!"], 3, 0, "Typed")
 *
 *
 */

 var PopUpParameters = PluginManager.parameters('Popup');
 var PopUpDefaultTimer = Number(PopUpParameters["Popup Window Default Timer"] || 5);
 var PopUpNoteDefaultOpacity = Number(PopUpParameters["Item Notification Default Opacity"] || 0);
 var PopUpNoteDefaultStyle = PopUpParameters["Item Notification Default Style"] || "Typed";
 var PopUpNoteMessage = PopUpParameters["Item Notification Message"] || "amountx icon item";
 var PopUpAllowItemPopup = PopUpParameters["Autmoatic Popup on Gain Item"] || "False";
 var PopUpAbovePlayerDefault = PopUpParameters["Display Position"] || "Player";
 var PopUpItemPosition = PopUpParameters["Item Display Position"] || "Player";
 
 
function Window_Popup() {
	this.initialize.apply(this, arguments);
}
Window_Popup.prototype = Object.create(Window_Base.prototype);
Window_Popup.prototype.constructor = Window_Popup;
Window_Popup.prototype.initialize = function() {
	Window_Base.prototype.initialize.call(this, 0, 0, 48, 48);
	this.available = true;
	this.openness -= 30;
	this.close();
}
Window_Popup.prototype.set = function(texts, timer, opacity, style, x, y, position) {
	this.yy = 0;
	var width = 0;
	this.opacity = opacity == null ? 100 : opacity;
	this.style = style == null ? "Normal" : style;
	timer = timer == null ? PopUpDefaultTimer : timer;
	for(var i = 0;i < texts.length;i++) {
		var tempString = this.convertEscapeCharacters(texts[i]);
		tempString = texts[i].replace(/[^invpgINVPG]\[\d{0,3}\]/g, "");
		tempString = texts[i].replace(/[qQ]\[.{6}\]/g, "");
		tempString = tempString.replace(/[iI]\[\d{0,3}\]/g, this.iconWidth());
		tempString = tempString.replace(/\\/g, "");
		if(width < this.textWidth(tempString) + 48) { width = this.textWidth(tempString) + 48; }
	}
	var height = this.lineHeight() * texts.length + this.lineHeight();
	this.width = width;
	this.height = height;
	this.popupPosition = position;
	this.offX = x || 0;
	this.offY = y || 0;
	if(this.popupPosition == "player") {
		this.updatePosition();
	} else {
		this.initPosition();
	}
	this.texts = texts;
	this.timer = timer * 60;
	this.closeTimer = 30;
	this.available = false;
	this.doneDrawing = false;
	this.createContents();
	this.open();
}

Window_Popup.prototype.updateRefresh = function() {
	var yy = 0;
	for(var i = 0;i < this.texts.length;i++) {
		this.drawTextEx(this.texts[i], 0, yy);
		yy += this.lineHeight();
	}
	this.doneDrawing = true;
}

Window_Popup.prototype.update = function() {
	Window_Base.prototype.update.call(this);
	this.updatePosition();
	if(this.available) { return; }
	if(this.doneDrawing) {
		this.timer--;
		if(this.timer <= 0) {
			this.available = true;
			this.close();
		}
	} else {
		if(this.style == "Normal") { this.updateRefresh(); }
		if(this.style == "Typed") { this.updateCharacter(); }
	}
}
Window_Popup.prototype.updatePosition = function() {
	if(this.popupPosition == "player") {
		this.x = $gamePlayer.screenX() - this.width / 2 + this.offX;
		this.y = $gamePlayer.screenY() - 120 + this.offY;
		if(this.y < 0) { this.y = 0; }
	}
}
Window_Popup.prototype.initPosition = function() {
	this.x = (Graphics.width - this.width) / 2;
	this.y = (Graphics.height - this.height) / 2;
	switch(this.popupPosition) {
		case "top":
			this.y = 50;
			break;
		case "right":
			this.x = Graphics.width - this.width;
			break;
		case "bottom":
			this.y = Graphics.height - this.height;
			break;
		case "left":
			this.x = 0;
			break;
		case "topleft":
			this.x = 0;
			this.y = 0;
			break;
		case "topright":
			this.x = Graphics.width - this.width;
			this.y = 0;
			break;
		case "bottomright":
			this.x = Graphics.width - this.width;
			this.y = Graphics.height - this.height;
			break;
		case "bottomleft":
			this.x = 0;
			this.y = Graphics.height - this.height;
			break;
		default:
			break;
	}
	this.x += this.offX;
	this.y += this.offY;
}
Window_Popup.prototype.isAvailable = function() {
	return this.available && this.isClosed();
}
Window_Popup.prototype.iconWidth = function() {
	var size = this.textWidth(" ");
	var string = " ";
	for(var i = 0;i < 24/size;i++) {
		string += " ";
	}
	return string;
}
Window_Popup.prototype.updateCharacter = function() {
	if(this.textState == null) { 
		if(this.texts.length > 0) {
			this.textState = { index: 0, x: 0, y: this.yy, left: 0, timer: 0};
			this.textState.text = this.convertEscapeCharacters(this.getNextText());
			this.textState.height = this.calcTextHeight(this.textState, false);
		} else {
			this.doneDrawing = true;
			return;
		}
	}
	this.textState.timer++;
	if(this.textState.timer > 1) {
		if(this.textState.index < this.textState.text.length) {
			this.processCharacter(this.textState);
			this.textState.timer = 0;
		} else {
			this.textState = null;
			this.yy += this.lineHeight();
		}
	}
}
Window_Popup.prototype.getNextText = function() {
	this.texts.reverse();
	var text = this.texts.pop();
	this.texts.reverse();
	return text;
}
Window_Popup.prototype.processEscapeCharacter = function(code, textState) {
	Window_Base.prototype.processEscapeCharacter.call(this, code, textState);
	if(code == "Q") {
		this.changeTextColor("#" + this.obtainEscapeParamSpecial(textState));
	}
};
Window_Popup.prototype.obtainEscapeParamSpecial = function(textState) {
    var arr = /^\[.+\]/.exec(textState.text.slice(textState.index));
    if (arr) {
        textState.index += arr[0].length;
        return arr[0].slice(1).slice(0, -1);
    } else {
        return '';
    }
};

function Popup() {}
Popup.init = function() { this.queue = []; }
Popup.add = function(texts, timer, opacity, style, x, y, position) {
	if(this.queue == null) { Popup.init(); }
	this.queue.push([texts, timer, opacity, style, x, y, PopUpAbovePlayerDefault.toLowerCase()]);
}
Popup.addItem = function(item, amount) {
	if(this.queue == null) { Popup.init(); }
	if(item) {
		var text = PopUpNoteMessage.replace(/icon/, "\\i[" + item.iconIndex + "]");
		if(item._color) {
			text = text.replace(/item/, "\\q[" + item._color + "]" + item.name);
		} else {
			text = text.replace(/item/, item.name);
		}
		text = text.replace(/amount/, amount);
		this.queue.push([[text], null, PopUpNoteDefaultOpacity, PopUpNoteDefaultStyle, null, null, PopUpItemPosition.toLowerCase()]);
	}
}
Popup.getQueue = function() { 
	if(this.queue == null) { Popup.init(); }
	return this.queue; 
}
Popup.getNext = function() {
	this.queue.reverse();
	var item = this.queue.pop();
	this.queue.reverse();
	return item;
}

var PopUpSceneMapCreate = Scene_Map.prototype.createAllWindows;
var PopUpSceneMapUpdate = Scene_Map.prototype.update;

Scene_Map.prototype.createAllWindows = function() {
	PopUpSceneMapCreate.call(this);
	this.popupWindow = new Window_Popup();
	this.addWindow(this.popupWindow);
}
Scene_Map.prototype.update = function() {
	PopUpSceneMapUpdate.call(this);
	if(this.popupWindow != null) { this.updatePopupWindow(); }
}
Scene_Map.prototype.popupActive = function() {
	return this.popupWindow.isOpen();
}
Scene_Map.prototype.updatePopupWindow = function() {
	if(this.popupWindow.isAvailable()) {
		if(Popup.getQueue().length > 0) {
			var args = Popup.getNext();
			this.popupWindow.set(args[0],args[1],args[2],args[3],args[4],args[5],args[6]);
		}
	} else {
		if(Input.isTriggered('') || Input.isTriggered('')) {
			this.popupWindow.timer = 0;
		}
		this.popupWindow.update();
	}
}

var PopUpGamePartyGainItem = Game_Party.prototype.gainItem;
Game_Party.prototype.gainItem = function(item, amount, includeEquip) {
    PopUpGamePartyGainItem.call(this, item, amount, includeEquip);
	if(amount > 0 && PopUpAllowItemPopup.toLowerCase() == "true") { Popup.addItem(item, amount); }
};

var PopUpGamePlayerStartMapEvent = Game_Player.prototype.startMapEvent;
Game_Player.prototype.startMapEvent = function(x, y, triggers, normal) {
    if(SceneManager._scene.popupActive()) { return; }
	PopUpGamePlayerStartMapEvent.call(this, x, y, triggers, normal);
};

var Popup_Game_Interpreter_pluginCommand = Game_Interpreter.prototype.pluginCommand;
Game_Interpreter.prototype.pluginCommand = function(command, args) {
	Popup_Game_Interpreter_pluginCommand.call(this, command, args);
	if(command == "Popup".toLowerCase()) {
		if(args[0] === 'position') {
			PopUpAbovePlayerDefault = args[1];
		} else if (args[0] === 'automaticPopup') {
			PopUpAllowItemPopup = args[0];
		}
	}
}



















